# dowloading and cleaning suicide profile
rm(list=ls())
library(fingertipsR)
# important: version 0.2.9 is used below, later version have different implementation for accessing the data

# load remaining packages
library(tidyverse)
library(ggridges)
library(WVPlots)
library(stringr)

## identify suicide profiles
profiles <- profiles()
sui_profiles <- profiles %>% filter(str_detect(ProfileName, "[Ss]uicide"))

## download profile data
dataset <- fingertips_data(ProfileID = 91)
save(dataset, file="fingertipsDataRaw0.Rdata")
load("fingertipsDataRaw0.Rdata")

## download profile data
unique(dataset[,2])

# filter for right suicide response variable
idRemove <- which(dataset[,2] == "Suicide crude rate 10-34 years: per 100,000 (5 year average)")                                                
dataset  <- dataset[-idRemove,]
idRemove <- which(dataset[,2] == "Suicide crude rate 35-64 years: per 100,000 (5 year average)")
dataset  <- dataset[-idRemove,]
idRemove <- which(dataset[,2] == "Years of life lost due to suicide, age-standardised rate 15-74 years: per 10,000 population (3 year average)")
dataset  <- dataset[-idRemove,]
idRemove <- which(dataset[,2] == "Suicide crude rate 65+ years: per 100,000  (5 year average)")                                     
dataset  <- dataset[-idRemove,]
save(dataset, file="fingertipsDataRaw1.Rdata")
load("fingertipsDataRaw1.Rdata")

# relabel covariates to simpler and shorter names.
dataset[which(dataset[,2] == "Social Isolation: percentage of adult social care users who have as much social contact as they would like"), 2] <- "social_isolation_cares"
dataset[which(dataset[,2] == "Social Isolation: percentage of adult carers who have as much social contact as they would like"), 2]  <- "social_isolation_users"
dataset[which(dataset[,2] == "Statutory homelessness - households in temporary accommodation"),         2] <- "homeless_temp_acco"
dataset[which(dataset[,2] == "Statutory homelessness - Eligible homeless people not in priority need"), 2] <- "homeless_nonpriority"
dataset[which(dataset[,2] == "Estimated prevalence of common mental disorders: % of population aged 16 & over"), 2] <- "mh_16plus"
dataset[which(dataset[,2] == "Estimated prevalence of common mental disorders: % of population aged 65 & over"), 2] <- "mh_65plus"
dataset[which(dataset[,2] == "Mental Health: QOF prevalence (all ages)"), 2] <- "mh_qof"
dataset[which(dataset[,2] == "Suicide rate"), 2]  <- "suicide_rate"
dataset[which(dataset[,2] == "Children in the youth justice system (10-18 yrs)"), 2] <- "children_youth_justice"
dataset[which(dataset[,2] == "children_leaving_care"), 2]  <- "Children leaving care: rate per 10,000 children aged under 18"
dataset[which(dataset[,2] == "Domestic abuse - historic method"), 2] <- "domestic_abuse"
dataset[which(dataset[,2] == "Depression: Recorded prevalence (aged 18+)"), 2]  <- "depression"
dataset[which(dataset[,2] == "Long term claimants of Jobseeker's Allowance"), 2] <- "lt_unepmloyment"
dataset[which(dataset[,2] == "Marital breakup: % of adults"), 2] <- "marital_breakup"
dataset[which(dataset[,2] == "Unemployment (model-based)"), 2] <- "unemployment"
dataset[which(dataset[,2] == "People living alone: % of all usual residents in households occupied by a single person"), 2] <- "alone"
dataset[which(dataset[,2] == "Older people living alone: % of households occupied by a single person aged 65 or over"), 2] <- "old_pople_alone"
dataset[which(dataset[,2] == "Children in care"), 2] <- "looked_after_children"
dataset[which(dataset[,2] == "Estimated prevalence of opiate and/or crack cocaine use"), 2] <- "opiates"
dataset[which(dataset[,2] == "Long-term health problem or disability: % of population"), 2] <- "lt_health_problems"
dataset[which(dataset[,2] == "Self-reported wellbeing - people with a low satisfaction score"), 2] <- "wellbeing_lowsatisfaction"
dataset[which(dataset[,2] == "Self-reported wellbeing - people with a low worthwhile score"), 2] <- "wellbeing_lowworthwhile"
dataset[which(dataset[,2] == "Self-reported wellbeing - people with a low happiness score"), 2] <- "wellbeing_lowhappiness"
dataset[which(dataset[,2] == "Self-reported wellbeing - people with a high anxiety score"), 2] <- "wellbeing_highanxiety"
dataset[which(dataset[,2] == "Successful completion of alcohol treatment"), 2] <- "alcohol_rx"
dataset[which(dataset[,2] == "Successful completion of drug treatment - opiate users"), 2] <- "drug_rx_opiate"
dataset[which(dataset[,2] == "Successful completion of drug treatment - non-opiate users"), 2]  <- "drug_rx_non_opiate"
dataset[which(dataset[,2] == "Emergency Hospital Admissions for Intentional Self-Harm"), 2] <- "self_harm"
dataset[which(dataset[,2] ==  "Children leaving care: rate per 10,000 children aged under 18" ), 2] <- "child_care_dropout"
dataset[which(dataset[,2] == "Adults in treatment at specialist drug misuse services: rate per 1000 population"), 2] <- "drug_misuse_treatment"
dataset[which(dataset[,2] == "Adults in treatment at specialist alcohol misuse services: rate per 1000 population"), 2] <- "alcohol_misuse_treatment"
dataset[which(dataset[,2] == "Admission episodes for alcohol-related conditions (Broad)"), 2] <- "admission_alcohol"
dataset[which(dataset[,2] == "Domestic abuse-related incidents and crimes"), 2] <- "domestic_abuse"
save(dataset, file="fingertipsDataRaw2.Rdata")
load("fingertipsDataRaw2.Rdata")


# additional manipulation for data sorting and formatting
year2vars    <- table(dataset[,2], dataset[,24])
idSuicide    <- which(rownames(year2vars) == "suicide_rate")
idYears      <- names(which(year2vars[idSuicide,] == 0))
dataset      <- dataset[-which(dataset[,24] %in% idYears),]
data         <- dataset[,c(24, 2, 6, 8, 9, 13)] 
data         <- data[which(rowSums(is.na(data)) == 0),]
year2vars    <- table(data[,1], data[,2])
onceOnly     <- names(which(colSums(year2vars != 0) == 1)) 
data         <- data[-which(data[,2] %in% onceOnly),] 
year2vars    <- table(data[,1], data[,2])
suicideOnly  <- names(which(rowSums(year2vars != 0) == 1)) 
data         <- data[-which(data[,1] %in% suicideOnly),] 
data         <- data[which(data[,4] == "Persons"),] 

# remove superflous variable
data  <- data[,-c(4, 5)]
cNames <- c("year", "area", sort(unique(data[,2])))

# save intermediate data
save(data, file="fingertipsDataRaw3.Rdata")
load("fingertipsDataRaw3.Rdata")

# more formatting and sorting of data
dataMat <- matrix(nrow=0, ncol=length(cNames))
years   <- sort(unique(data[,1]))
areas   <- sort(unique(data[,3]))
covs    <- sort(unique(data[,2]))
for (year in years){
	print(year)
	for (area in areas){
		slh <- c(year, area)
		for (cov in 1:length(covs)){
			idY <- which(data[,1] == year)
			idA <- which(data[,3] == area)
			idC <- which(data[,2] == covs[cov])
			id  <- intersect(intersect(idY, idA), idC)
			if (length(id) >= 1){
				slh <- c(slh, data[id[1],4])
			} else {
				slh <- c(slh, NA)
			}
		}
		dataMat <- rbind(dataMat, slh)
	}
}
dataMat[,1]       <- as.numeric(dataMat[,1]) / 10000
rownames(dataMat) <- paste(dataMat[,2],  dataMat[,1], sep="_")
colnames(dataMat) <- cNames
dataMat           <- dataMat[,-2]
data              <- matrix(as.numeric(dataMat[,1]), ncol=1)
for (j in 2:ncol(dataMat)){
	X    <- as.numeric(dataMat[,j])
	data <- cbind(data, (X - median(X, na.rm=TRUE))/ mad(X, na.rm=TRUE))
}
rownames(data) <- rownames(dataMat)
colnames(data) <- colnames(dataMat)
data           <- data[,-c(20)]
cMat           <- cor(data[,-1], use="p")

# save data
save(data, file="fingertipsDataFinal.Rdata")
load("fingertipsDataFinal.Rdata")

# also obtain standardized data
data2 <- data
years <- sort(unique(data[,1]))
for (year in years){
	idY <- which(data2[,1] == year)
	for (j in 2:ncol(data2)){
		X    <- as.numeric(data2[idY,j])
		data2[idY,j] <- (X - mean(X, na.rm=TRUE))/ sd(X, na.rm=TRUE)
	}
}
save(data, data2, file="fingertipsDataFinal.Rdata")






