# remove all in memory
rm(list=ls())

library(MASS)

# declare functions
source("0_functions4analysis.r")

# load data
load("fingertipsDataFinal.Rdata")
data <- data2

years           <- sort(unique(data[,1]))
bHats           <- matrix(NA, ncol=ncol(data)-2, nrow=length(years)+1)
bHats[1,]       <- 0
rownames(bHats)	<- c("init", years)
colnames(bHats)	<- colnames(data)[-c(1,20)]
bPres           <- bHats
bRidge          <- bHats
bRidgeP         <- bHats
Xpast           <- matrix(nrow=0, ncol=ncol(bHats))
Ypast 		<- numeric()
optLambdasR     <- numeric()
optLambdasRp     <- numeric()
resLM <- numeric()
resRR <- numeric()
resRRp <- numeric()
for (y in 1:length(years)){
	year        <- years[y]
	idY         <- which(data[,1] == year)
	dataY       <- data[idY,]
	Y           <- dataY[,20]
	X           <- dataY[,-c(1,20)]
	colnames(X) <- NULL
	rownames(X) <- NULL
	Xslh        <- X
	idXremove   <- as.integer(which(apply(is.na(X), 2, sum) >= (nrow(dataY)-1)))
	idXkeep     <- as.integer(which(apply(is.na(X), 2, sum) <  (nrow(dataY)-1)))
	if (length(idXremove) > 0){
		X                 <- X[, -idXremove, drop=FALSE]
		Xslh[, idXremove] <- 0
	}
	idSremove <- which(apply(is.na(X), 1, sum) > 0)
	idSremove <- unique(c(idSremove, which(is.na(Y))))
	if (length(idSremove) > 0){
		X    <- data.matrix(X[-idSremove, , drop=FALSE])
		Xslh <- data.matrix(Xslh[-idSremove, , drop=FALSE])
		Y    <- as.numeric(Y[-idSremove])
	}		
	print(dim(X))	
	bHats[y+1,idXkeep]   <- as.numeric(solve(t(X) %*% X) %*% t(X) %*% Y)
	bHats[y+1,idXremove] <- bHats[y,idXremove]
	bPres[y+1,idXkeep]   <- 1
	bPres[y+1,idXremove] <- 0

	# targeted ridge (note the maximum on the penalty parameter)
	optLambda             <- optPenaltyLM.kCVauto(Y, X, lambdaInit=1, fold=length(Y), target=as.numeric(bRidge[y,idXkeep]))
	# bRidge[y+1,idXkeep]   <- as.numeric(solve(t(X) %*% X + optLambda*diag(ncol(X))) %*% t(X) %*% Y)
	bRidge[y+1,idXkeep]   <- ridgeLM(Y, X, optLambda, as.numeric(bRidge[y,idXkeep]))
	bRidge[y+1,idXremove] <- bRidge[y,idXremove]
	optLambdasR            <- c(optLambdasR, optLambda)

	# targeted ridge with constrained by fast fit
	if (y == 1){ 
		optLambda <- optPenaltyLM.kCVauto(Y, X, lambdaInit=1, fold=length(Y), target=as.numeric(bRidge[y,idXkeep]))
		# bRidgeP[y+1,idXkeep]   <- as.numeric(solve(t(X) %*% X + optLambda*diag(ncol(X))) %*% t(X) %*% Y)
		bRidgeP[y+1,idXkeep]   <- ridgeLM(Y, X, optLambda, as.numeric(bRidge[y,idXkeep]))
		bRidgeP[y+1,idXremove] <- bRidgeP[y,idXremove]
	} else {
		optLambda <- optPenaltyLMpandp.kCVauto(Y, Xslh, lambdaInit=1, fold=length(Y), target=bRidgeP[y,], Ypast, Xpast)
		bRidgeP[y+1,]   <- as.numeric(solve(t(Xslh) %*% Xslh + optLambda*diag(ncol(Xslh))) %*% t(Xslh) %*% Y)
		bRidgeP[y+1,]   <- ridgeLM(Y, Xslh, optLambda, as.numeric(bRidgeP[y,]))
		# bRidgeP[y+1,idXremove] <- bRidge[y,idXremove]
	}
	optLambdasRp <- c(optLambdasRp, optLambda)

	resLM  <- rbind(resLM,  cbind(rep(year, length(Y)), as.numeric(Y - Xslh %*% t(bHats[y+1,,drop=FALSE]))))
	resRR  <- rbind(resRR,  cbind(rep(year, length(Y)), as.numeric(Y - Xslh %*% t(bRidge[y+1,,drop=FALSE]))))
	resRRp <- rbind(resRRp, cbind(rep(year, length(Y)), as.numeric(Y - Xslh %*% t(bRidgeP[y+1,,drop=FALSE]))))

	Ypast <- c(Ypast, Y)
	Xpast <- rbind(Xpast, Xslh)
}
save(resLM, resRR, resRRp, file="residuals.Rdata")



# plots
ylims <- c(min(bHats), max(bHats))
plot(bHats[,1] ~ c(2007, years), col="white", ylim=ylims, ylab=expression(hat(beta)[j]), axes=FALSE, xlab="year")
axis(2, srt=90)
axis(1, labels=c("init", years), at=c(2007, years))
for (cov in 1:ncol(bHats)){
	lines(bHats[,cov] ~ c(2007, years), col=cov, lty=cov, lwd=2)
	points(bHats[,cov][bPres[,cov]==1] ~ c(2007, years)[bPres[,cov]==1], col=cov, pch=cov)
}

ylims <- c(min(bRidge), max(bRidge))
plot(bHats[,1] ~ c(2007, years), col="white", ylim=ylims, ylab=expression(hat(beta)[j]), axes=FALSE, xlab="year")
axis(2)
axis(1, labels=c("init", years), at=c(2007, years))
for (cov in 1:ncol(bRidge)){
	lines(bRidge[,cov] ~ c(2007, years), col=cov, lty=cov, lwd=2)
	points(bRidge[,cov][bPres[,cov]==1] ~ c(2007, years)[bPres[,cov]==1], col=cov, pch=cov)
}

ylims <- c(min(bRidgeP), max(bRidgeP))
plot(bRidgeP[,1] ~ c(2007, years), col="white", ylim=ylims, ylab=expression(hat(beta)[j]), axes=FALSE, xlab="year")
axis(2)
axis(1, labels=c("init", years), at=c(2007, years))
for (cov in 1:ncol(bRidgeP)){
	lines(bRidgeP[,cov] ~ c(2007, years), col=cov, lty=cov, lwd=2)
	points(bRidgeP[,cov][bPres[,cov]==1] ~ c(2007, years)[bPres[,cov]==1], col=cov, pch=cov)
}


# plot some individuals regression paths
id <- 12 
plot(bHats[,id] ~ c(2007, years), type="l", lwd=2, axes=FALSE, xlab="year", ylab="estimated effect of self-harm")
axis(2)
axis(1, labels=c("init", years), at=c(2007, years))
lines(bRidge[,id] ~ c(2007, years), col="red", lwd=2, lty=2)
lines(bRidgeP[,id] ~ c(2007, years), col="purple", lwd=2, lty=4)
legend("bottomleft", c("OLS", "ridge, target", "ridge, target, constrain. CV"), lwd=2, col=c("black", "red", "blue"), lty=c(1,2, 4))

id <- 1
plot(bHats[,id] ~ c(2007, years), type="l", lwd=2, axes=FALSE, xlab="year", ylab="estimated effect of alcohol-related issues")
axis(2)
axis(1, labels=c("init", years), at=c(2007, years))
lines(bRidge[,id] ~ c(2007, years), col="red", lwd=2, lty=2)
lines(bRidgeP[,id] ~ c(2007, years), col="purple", lwd=2, lty=4)
legend("bottomleft", c("OLS", "ridge, target", "ridge, target, constrain. CV"), lwd=2, col=c("black", "red", "blue"), lty=c(1,2, 4))


