# preliminaries
rm(list=ls())
gc()

# load libraries
library(mvtnorm)
library(porridge)
library(microbenchmark)

# set sample size (vary sample size)
n <- 25

# set true parameter (vary size)
betas <- (c(0:1000) - 500) / 2000

# generate pilot data
X     <- matrix(rnorm(length(betas)*n), nrow=n)
probs <- exp(X %*% betas) / (1 + exp(X %*% betas))
Y     <- numeric()
for (i in 1:n){
	Y <- c(Y, sample(c(0,1), 1, prob=c(1-probs[i], probs[i])))
}

# set value of the penalty parameter (try various values)
lambda <- 100

# evaluate computing times
timings <- microbenchmark(ridgeGLM(Y, X, lambda, model="logistic", target=0*betas),
                          ridgeGLM(Y, X, lambda, model="logistic", target=betas),
                          ridgeGLM(Y, X, lambda, model="logistic", target=0.5*betas),
                          ridgeGLM(Y, X, lambda, model="logistic", target=2*betas),
                          ridgeGLM(Y, X, lambda, model="logistic", target=-1*betas),
                          ridgeGLM(Y, X, lambda, model="logistic", target=rnorm(length(betas))))

# extract times
calls <- levels(timings$expr)[timings$expr]
calls <- substr(calls, 53, nchar(calls)-1)
timings <- timings$time
times <- timings[calls == "0 * betas"]
times <- cbind(times, timings[calls == "0.5 * betas"])
times <- cbind(times, timings[calls == "betas"])
times <- cbind(times, timings[calls == "2 * betas"])
times <- cbind(times, timings[calls == "-1 * betas"])
times <- cbind(times, timings[calls == "rnorm(length(betas))"])
times <- log(times)

# make boxplot of computing times with various targets
ylims <- c(min(times), 14.0)
xlims <- c(0,6)
op <- par(mar=c(6.1,4.1,4.1,2.1), pty = "s") 
boxplot(times[,1], at=1, xlims=xlims, ylims=ylims, col="white", border="white", ylab="log(times)",
	main=expression(paste("IRLS comp. times vs. target; n=25, p=1000, ", lambda, "=100", sep="")), axes=FALSE)
axis(2)
ats <- c(0.5, 1.5, 2.5, 3.5, 4.5, 5.5)
axis(1, at=ats,  labels=FALSE)
bpLabs <- c(expression(paste(beta[0], " = ",    0[p], sep="")), 
            expression(paste(beta[0], " = ", beta, "/2", sep="")),
            expression(paste(beta[0], " = ",    beta, sep="")),
            expression(paste(beta[0], " = 2",  beta, sep="")),
            expression(paste(beta[0], " = -",  beta, sep="")),
            expression(paste(beta[0], " = random", sep="")))
text(x=ats, y=par()$usr[3]-0.1*(par()$usr[4]-1.3*par()$usr[3]+3.515), labels=bpLabs, srt=45, adj=1, xpd=TRUE)
cols=c("brown", "blue", "purple", "red", "orange", "yellow")
for (u in 1:ncol(times)){
	boxplot(times[,u], col=cols[u], at=ats[u], add=TRUE, pch=20)
}

