# preliminaries
rm(list=ls())
gc()

# load libraries
library(mvtnorm)
library(penalized)
library(porridge)
library(Matrix)


# set dimension and sample size
p <- 101
n <- 25

# regression parameter
betas <- seq(-2.5, 2.5, length.out=p)

# generate banded precision matrix
diags <- list(rep(1, p), rep(0.5, p-1), rep(0.25, p-2), rep(0.1, p-3))
Sigma <- as.matrix(bandSparse(p, k = -c(0:3), diag = c(diags), symm=TRUE))
colnames(Sigma) <- rownames(Sigma) <- NULL

# create storage object
ncRR1resList <- list()

for (u in 1:100){
	print(u)

	# generate pilot data
	X <- matrix(rnorm(length(betas)*n), nrow=n)
	# X <- rmvnorm(n, sigma=Sigma)
	# X <- sweep(X, 2, runif(ncol(X), -10, 10), FUN="+")
	Y <- X %*% betas + rnorm(n, sd=0.2)

	# specify target
	# replace this by other targets
	bHat <- -betas

	# initiate result storage
	ncRRres1 <- matrix(c(0, bHat, 10, sum((betas - bHat)^2)), nrow=1)
	colnames(ncRRres1) <- c("time", paste("b", 1:length(betas), sep=""), "lambdaS", "bias")
	for (k in 1:25){
		nlmRes <- list()
		nlmRes$convergence <- 1
		optLambdas <- c(-1)
		X <- matrix(rnorm(length(betas)*n), nrow=n)
		X <- sweep(X, 2, runif(ncol(X), -10, 10), FUN="+")
		Y <- X %*% betas + rnorm(n, sd=0.2)

		optLambda <- optPenaltyGLM.kCVauto(Y, X, lambdaInit=1, fold=10, 
                                        stratified=TRUE, loss="loglik",
                                       	target=bHat, model="linear")

		bHat <- ridgeGLM(Y, X, lambda=optLambda, model="linear", target=bHat)
		ncRRres1 <- rbind(ncRRres1, c(k, bHat, optLambdas, sum((betas - bHat)^2)))
	}
	ncRR1resList[[u]] <- ncRRres1
}



####################################################################################################


# start plot
slh <- numeric()
for (u in 1:length(ncRR1resList)){
	slh <- cbind(slh, ncRR1resList[[u]][,2])
}
quants <- apply(slh, 1, quantile, probs=c(0.05, 0.5, 0.95), na.rm=TRUE)
times <- c(0:(nrow(slh)-1))
plot(quants[2,] ~ times, type="l", col="blue", lwd=2, ylab=expression(beta), xlab="update",  ylim=c(-3.0, 4.0), main="Targeted ridge estimates, X nonzero-centered and collinear")
lines(rep(-2.5, length(times)) ~ times, col="grey", lwd=2, lty=3)
lines(rep(-1, length(times)) ~ times, col="grey", lwd=2, lty=3)
lines(rep(0, length(times)) ~ times, col="grey", lwd=2, lty=3)
lines(rep(1, length(times)) ~ times, col="grey", lwd=2, lty=3)
lines(rep(2.5, length(times)) ~ times, col="grey", lwd=2, lty=3)
revOrder <- sort(1:length(times), decreasing=TRUE)
polygon(c(times, times[revOrder]), c(quants[1,], quants[3,revOrder]), col="blue", density=50, border=NA)
lines(quants[1,] ~ times, lty=2, col="blue", lwd=2)
lines(quants[3,] ~ times, lty=2, col="blue", lwd=2)

slh <- numeric()
for (u in 1:length(ncRR1resList)){
	slh <- cbind(slh, ncRR1resList[[u]][,32])
}
quants <- apply(slh, 1, quantile, probs=c(0.05, 0.5, 0.95), na.rm=TRUE)
times <- c(0:(nrow(slh)-1))
revOrder <- sort(1:length(times), decreasing=TRUE)
lines(quants[2,] ~ times, lty=1, col="purple", lwd=2)
polygon(c(times, times[revOrder]), c(quants[1,], quants[3,revOrder]), col="purple", density=50, border=NA)
lines(quants[1,] ~ times, lty=2, col="purple", lwd=2)
lines(quants[3,] ~ times, lty=2, col="purple", lwd=2)

slh <- numeric()
for (u in 1:length(ncRR1resList)){
	slh <- cbind(slh, ncRR1resList[[u]][,52])
}
quants <- apply(slh, 1, quantile, probs=c(0.05, 0.5, 0.95), na.rm=TRUE)
times <- c(0:(nrow(slh)-1))
revOrder <- sort(1:length(times), decreasing=TRUE)
lines(quants[2,] ~ times, lty=1, col="red", lwd=2)
polygon(c(times, times[revOrder]), c(quants[1,], quants[3,revOrder]), col="red", density=50, border=NA)
lines(quants[1,] ~ times, lty=2, col="red", lwd=2)
lines(quants[3,] ~ times, lty=2, col="red", lwd=2)

slh <- numeric()
for (u in 1:length(ncRR1resList)){
	slh <- cbind(slh, ncRR1resList[[u]][,72])
}
quants <- apply(slh, 1, quantile, probs=c(0.05, 0.5, 0.95), na.rm=TRUE)
times <- c(0:(nrow(slh)-1))
revOrder <- sort(1:length(times), decreasing=TRUE)
lines(quants[2,] ~ times, lty=1, col="orange", lwd=2)
polygon(c(times, times[revOrder]), c(quants[1,], quants[3,revOrder]), col="orange", density=50, border=NA)
lines(quants[1,] ~ times, lty=2, col="orange", lwd=2)
lines(quants[3,] ~ times, lty=2, col="orange", lwd=2)

slh <- numeric()
for (u in 1:length(ncRR1resList)){
	slh <- cbind(slh, ncRR1resList[[u]][,102])
}
quants <- apply(slh, 1, quantile, probs=c(0.05, 0.5, 0.95), na.rm=TRUE)
times <- c(0:(nrow(slh)-1))
revOrder <- sort(1:length(times), decreasing=TRUE)
lines(quants[2,] ~ times, lty=1, col="yellow", lwd=2)
polygon(c(times, times[revOrder]), c(quants[1,], quants[3,revOrder]), col="yellow", density=50, border=NA)
lines(quants[1,] ~ times, lty=2, col="yellow", lwd=2)
lines(quants[3,] ~ times, lty=2, col="yellow", lwd=2)

legend("topleft", c(expression(paste(beta, "=2.5", sep="")), expression(paste(beta, "=1", sep="")), expression(paste(beta, "=0", sep="")), expression(paste(beta, "=-1", sep="")), expression(paste(beta, "=-2.5", sep=""))), lwd=2, col=c("yellow", "orange", "red", "purple", "blue"))


